/*
 * fc-config-override
 * Fontconfig Configuration Override
 *
 * This preloaded wrapper library allows the use of an
 * alternate Fontconfig configuration file on a per-application
 * basis in cases where setting the FC_CONFIG_FILE environment
 * variable doesn't work.  I use this to force the use of
 * non-antialiased bitmap fonts in the GNOME Terminal.
 *
 * Usage:
 *   LD_PRELOAD=/path/to/fc-config-override.so your_application
 *
 * This program is in the public domain - do whatever you want with it.
 *
 * David Simmons
 * August 24, 2009
 *
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>

void open_test(int result) {
	if (result == -1) {
		perror("\topen failed");
	} else {
		printf("\topen succeeded fd=%d\n", result);
		close(result);
	}
}

void open_without_mode(const char *pathname, int flags) {
	printf("open(\"%s\", %08X);\n", pathname, flags);
	open_test(open(pathname, flags));
}

void open_with_mode(const char *pathname, int flags, mode_t mode) {
	printf("open(\"%s\", %08X, 0%o);\n", pathname, flags, mode);
	open_test(open(pathname, flags, mode));
}

int main(int argc, char **argv) {
	char filename[64];

	// enable debugging in the wrapper
	setenv("FC_WRAPPER_DEBUG", "1", 1);

	open_without_mode("/etc/resolv.conf", O_RDONLY);
	open_without_mode("/etc/motd", O_RDONLY);
	open_without_mode("/etc/file_not_found", O_RDONLY);
	open_without_mode("/etc/fonts/fonts.conf", O_RDONLY);

	snprintf(filename, 64, "/tmp/fc-wrapper-test-%d", getpid());
	open_with_mode(filename, O_WRONLY|O_CREAT, 0644);
	unlink(filename);

	return 0;
}

